#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##############################################################################
# IRKit command line interface
# -----------------------------------------------
# $Revision: 83 $
# $Id: irkit.py 83 2016-12-03 08:49:56Z  $
#
# Copyright (C) 2015-2016 Yoshihiro Naito. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
##############################################################################
import argparse
import json
import os
import sys
import urllib.request
from resolve_irkit import resolve_irkits
from socket import gethostbyname

Version = '0.2.7'
__author__ = 'Y.Naito'
__date__ = '$Date: 2016-12-03 17:49:56 +0900 (土, 03 12 2016) $'
__version__ = '$Id: irkit.py 83 2016-12-03 08:49:56Z  $'


def ir_save_dir():
    """Save directory of IR signal files"""
    return os.path.expanduser('~') + '/.irkit.d'


save_dir = ir_save_dir()  # Getした赤外線信号の保存先ディレクトリ名
make_filename = lambda signal_name: save_dir + '/signals/' + signal_name + '.json'


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Control IRKit')
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s '+Version)
    parser.add_argument('-V', '--verbose', action='store_true',
                        default=False, help='verbose mode')
    subparsers = parser.add_subparsers(dest='command')
    # create the parser for the "get" command
    parser_get = subparsers.add_parser('get', aliases=['read'],
                                       help='GET command')
    parser_get.add_argument('ir_signal_name', action='store',
                            help='IR signal name')
    parser_get.add_argument('-a', '--address',
                            nargs=1,
                            action='store',
                            default=None,
                            dest='host',
                            metavar='address',
                            help='IP address of IRKit device')
    # create the parser for the "post" command
    parser_post = subparsers.add_parser('post', aliases=['send'],
                                        help='POST command')
    parser_post.add_argument('ir_signal_name', action='store',
                             help='IR signal name')
    parser_post.add_argument('-a', '--address',
                             nargs=1,
                             action='store',
                             default=None,
                             dest='host',
                             metavar='address',
                             help='IP address of IRKit device')
    # create the parser for the "show" command
    parser_show = subparsers.add_parser('show', aliases=['view'],
                                        help='Show IR signal data')
    parser_show.add_argument('ir_signal_name', action='store',
                             help='IR signal name')
    # create the parser for the "list" command
    parser_list = subparsers.add_parser(
                      'list',
                      help='Print list saved IR signal'
                  )
    # create the parser for the "rename" command
    parser_rename = subparsers.add_parser(
                        'rename',
                        aliases=['replace'],
                        help='Rename saved IR signal'
                    )
    parser_rename.add_argument('old_signal_name', action='store',
                               help='Old IR signal name')
    parser_rename.add_argument('new_signal_name', action='store',
                               help='New IR signal name')
    # create the parser for the "delete" command
    parser_delete = subparsers.add_parser(
                        'delete',
                        aliases=['remove'],
                        help='Delete saved IR signal'
                    )
    parser_delete.add_argument('ir_signal_name',
                               action='store',
                               help='IR signal name')
    #
    args = parser.parse_args()
    verbose = args.verbose
else:
    verbose = False


def print_message(msg):
    """Output error message to stderr"""
    if __name__ == '__main__':
        print(msg, file=sys.stderr)


class IRKit:
    """Irkit class"""
    def __init__(self, host, port):
        url = 'http://' + host + ':' + port + '/messages'
        self.request = urllib.request.Request(
                           url,
                           headers={'X-Requested-With': 'XMLHttpRequest'}
                       )

    def get(self):
        """最も新しい受信した赤外線信号を返す。"""
        with urllib.request.urlopen(url=self.request) as self.f:
            return self.f.read().decode('utf-8')

    def post(self, json_data):
        """赤外線信号を送信する。"""
        with urllib.request.urlopen(url=self.request,
                                    data=json_data.encode('utf-8')):
            pass


def check_dir(path):
    """pathディレクトリの有無を調べ、無い場合は作成する。"""
    if os.path.exists(path):
        assert os.path.isdir(path), path + ' is not directory.'
    else:
        os.mkdir(path)


def search_irkit():
    """BonjourでIRKitを探し、見つかったIRKitの情報を返却する。"""
    irkits = resolve_irkits()
    assert irkits, 'No IRKit device'
    irkit = irkits[0]    # 最初に見つかったIRKit
    host = irkit[0]
    address = gethostbyname(host)
    port = str(irkit[1])
    return {'Host': host, 'IPaddress': address, 'Port': port}


def get_irkit_info():
    """IRKit の情報辞書(IPアドレス,ポート番号,サーバ名)を返す。"""
    save_file = save_dir + '/irkit.save'
    try:
        with open(save_file, mode='rt', encoding='utf-8') as f:
            irkit_info = json.load(f)
    except:
        irkit_info = search_irkit()
        with open(save_file, mode='wt', encoding='utf-8') as f:
            json.dump(irkit_info, f)
    else:
        # Check IRKit info.

        if irkit_info['IPaddress'] != gethostbyname(irkit_info['Host']):
            # IP address has changed
            irkit_info = search_irkit()
            with open(save_file, mode='wt', encoding='utf-8') as f:
                json.dump(irkit_info, f)
    if verbose:
        print('IRKit device: {0}'.format(irkit_info))
    return irkit_info


def get(address, port, host, signal_name):
    """GET sub-command"""
    irkit = IRKit(address, port)
    data = irkit.get()
    if data:
        ir_data = json.loads(data)
        ir_data['name'] = signal_name
        ir_data['hostname'] = host.split('.')[0]
        ir_data['type'] = 'single'
        file_name = make_filename(signal_name)
        if verbose:
            print('{0} <= {1}'.format(file_name, ir_data))
        with open(file_name, mode='wt', encoding='utf-8') as f:
            json.dump(ir_data, f, indent=2, ensure_ascii=False,
                      separators=(',', ': '))
        return True
    else:
        print_message('No data')
        return False


def post(address, port, signal_name):
    """POST sub-command"""
    file_name = make_filename(signal_name)
    with open(file_name, mode='rt', encoding='utf-8') as f:
        ir_data = json.load(f)
        if verbose:
            print('{0} => {1}'.format(file_name, ir_data))
        irkit = IRKit(address, port)
        irkit.post(json.dumps(ir_data))


def show(signal_name):
    """Show sub-command"""
    file_name = make_filename(signal_name)
    with open(file_name, mode='rt', encoding='utf-8') as f:
        print('{0} = {1}'.format(file_name, json.load(f)))


def list(root):
    """List sub-command"""
    for path, dirs, files in os.walk(root):
        for file in sorted(files):
            if file[0] != '.':            # 先頭が'.'のファイルは除く
                if verbose:
                    print(os.path.join(path, file))
                else:
                    print(os.path.join(
                            path,
                            os.path.splitext(file)[0]).replace(root, '')
                          )


def rename(old_name, new_name):
    """Rename sub-command"""
    if old_name != new_name:
        old_file_name = make_filename(old_name)
        assert os.path.exists(old_file_name), \
            'The IR signal ' + signal_name + ' does not exist.'
        if verbose:
            print('rename '+old_name+' to '+new_name)
        with open(old_file_name, mode='rt', encoding='utf-8') as fin:
            ir_data = json.load(fin)
            ir_data['name'] = new_name
            with open(make_filename(new_name),
                      mode='wt', encoding='utf-8') as fout:
                json.dump(ir_data,
                          fout,
                          indent=2,
                          ensure_ascii=False,
                          separators=(',', ': '))
                os.remove(old_file_name)
    else:
        pass


def delete(signal_name):
    """Delete sub-command"""
    file_name = make_filename(signal_name)
    assert os.path.exists(file_name), \
        'The IR signal ' + signal_name + ' does not exist.'
    if verbose:
        print('delete '+file_name)
    os.remove(file_name)


def main():
    check_dir(save_dir)
    check_dir(save_dir + '/signals')

    if args.command == 'get' or args.command == 'read':
        if args.host:
            address, port = args.host[0], '80'
        else:
            info = get_irkit_info()
        get(info['IPaddress'], info['Port'], info['Host'], args.ir_signal_name)
    elif args.command == 'post' or args.command == 'send':
        if args.host:
            address, port = args.host[0], '80'
        else:
            info = get_irkit_info()
        post(info['IPaddress'], info['Port'], args.ir_signal_name)
    elif args.command == 'show' or args.command == 'view':
        show(args.ir_signal_name)
    elif args.command == 'list':
        list(save_dir + '/signals')
    elif args.command == 'rename' or args.command == 'replace':
        rename(args.old_signal_name, args.new_signal_name)
    elif args.command == 'delete' or args.command == 'remove':
        delete(args.ir_signal_name)
    else:
        pass


if __name__ == '__main__':
    main()
